from tests import IntegrationTest

class TestGetInstruments(IntegrationTest):

    def setUp(self):
        super().setUp()

    def test_add_instruments(self):            
        _json = self.get_instruments()
        assert len(_json['instruments']) == 3
        self.create_instrument('INSTR4')
        _json = self.get_instruments()
        assert len(_json['instruments']) == 4
        assert 'INSTR4' == _json['instruments'][3]['name']




from typing import Any, List

from popoll_backend.model import Payload


class Session(Payload):
    
    def __init__(self, data: List[Any]):
        self.id: int = data[0]
        self.session_id: str = data[1]
        self.user_id: int = data[2]

    @classmethod
    def create_table(cls):
        return """CREATE TABLE IF NOT EXISTS sessions (
            id integer PRIMARY KEY,
            session_id text NOT NULL,
            user_id integer NOT NULL REFERENCES users(id) ON DELETE CASCADE,
            datetime text NOT NULL
        );"""


from tests import IntegrationTest

class TestAnswersPerUser(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user2_id = self.create_user('user2', self.instru2_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.date2_id = self.create_date(date='2025-03-11', time='18:00:00', title='secondDate')
        self.date3_id = self.create_date(date='2025-03-12', time=None, title='thirdDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        self.answer2_id = self.create_answer(user_id=self.user1_id, date_id=self.date2_id)
        self.answer3_id = self.create_answer(user_id=self.user1_id, date_id=self.date3_id)
        self.answer4_id = self.create_answer(user_id=self.user2_id, date_id=self.date1_id)
        self.answer5_id = self.create_answer(user_id=self.user2_id, date_id=self.date2_id)
        self.answer6_id = self.create_answer(user_id=self.user2_id, date_id=self.date3_id)
        
    def test_get_answers_user(self):            
        _json = self.get_user(self.user1_id)
        assert _json['user']['user']['id'] == self.user1_id
        assert len(_json['dates']) == 3
        assert _json['dates'][0].get('answer', None) != None
        assert _json['dates'][0]['answer']['response'] == True
        
        assert _json['dates'][1].get('answer', None) != None
        assert _json['dates'][1]['answer']['response'] == True
        
        assert _json['dates'][2].get('answer', None) != None
        assert _json['dates'][2]['answer']['response'] == True

    # def test_get_answer_answerId(self):
    #     _json = self.toJSON(self.get_answer(self.answer1_id))
    #     assert len(_json['answers']) == 1
    #     assert _json['answers'][0]['user_id'] == self.user1_id
    #     assert _json['answers'][0]['date_id'] == self.date1_id
    #     assert _json['answers'][0]['response'] == True

    #     _json = self.toJSON(self.get_answer(999))
    #     assert len(_json['answers']) == 0


import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.model.db.date import Date
from popoll_backend.model.payload.dates_payload import DatesPayload
from popoll_backend.query import Query


class GetDates(Query):
    
    dates: List[Date]
    
    def __init__(self, poll: str):
        super().__init__(poll)

    def process(self, cursor: sqlite3.Cursor):
        self.dates = [Date(row) for row in cursor.execute('SELECT * FROM dates ORDER BY date, time').fetchall()]
        
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return DatesPayload(self.dates)


from tests import IntegrationTest

def _test_date_time(expectedDate, expectedTime, actualDate):
        assert expectedDate == actualDate['date']
        assert expectedTime == actualDate['time']

class TestGetDatesSortByDate(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user(name='user1', main_instrument=self.instru1_id, instruments=[])
        self.date1_id = self.create_date(date='2025-03-12', time='15:00:00', title='firstDate')
        self.create_answer(self.user1_id, self.date1_id)
        self.date2_id = self.create_date(date='2025-03-11', time='18:00:00', title='secondDate')
        self.create_answer(self.user1_id, self.date2_id)
        self.date3_id = self.create_date(date='2025-03-11', time=None, title='thirdDate')
        self.create_answer(self.user1_id, self.date3_id)
        
    def test_get_user_sorted(self):
        _json = self.get_user(self.user1_id)
        assert len(_json['dates']) == 3
        _test_date_time('2025-03-11', None, _json['dates'][0]['date'])
        _test_date_time('2025-03-11', '18:00:00', _json['dates'][1]['date'])
        _test_date_time('2025-03-12', '15:00:00', _json['dates'][2]['date'])



from tests import IntegrationTest

class TestGetUsersEntries(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user2_id = self.create_user('user2', self.instru2_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.date2_id = self.create_date(date='2025-03-11', time='18:00:00', title='secondDate')
        self.date3_id = self.create_date(date='2025-03-12', time=None, title='thirdDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        self.answer2_id = self.create_answer(user_id=self.user1_id, date_id=self.date2_id)
        self.answer3_id = self.create_answer(user_id=self.user1_id, date_id=self.date3_id)
        self.answer4_id = self.create_answer(user_id=self.user2_id, date_id=self.date1_id)
        self.answer5_id = self.create_answer(user_id=self.user2_id, date_id=self.date2_id)
        self.answer6_id = self.create_answer(user_id=self.user2_id, date_id=self.date3_id)
        
    def test_get_user(self):
        _json = self.get_users()
        assert len(_json['users']) == 2
        assert _json.get('dates', 'undefined') == 'undefined'
        assert _json.get('answers', 'undefined') == 'undefined'



from typing import Any, List

from popoll_backend.model import Payload


class UserInstruments(Payload):
    
    def __init__(self, data: List[Any]):
        self.id: int = data[0]
        self.user_id: int = data[1]
        self.instrument_id: int = data[2]
        self.is_main: bool = bool(data[3])

    @classmethod
    def create_table(cls):
        return """ CREATE TABLE IF NOT EXISTS user_instruments (
            id integer PRIMARY KEY,
            user_id integer NOT NULL REFERENCES users(id) ON DELETE CASCADE,
            instrument_id integer NOT NULL REFERENCES instruments(id) ON DELETE CASCADE,
            is_main boolean NOT NULL
        ); """


import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.payload.instruments_payload import InstrumentsPayload
from popoll_backend.query import Query


class GetInstruments(Query):
    
    instruments: List[Instrument]
    
    def __init__(self, poll: str):
        super().__init__(poll)
    
    def process(self, cursor: sqlite3.Cursor):
        self.instruments = [Instrument(row) for row in cursor.execute('SELECT * from instruments').fetchall()]
        
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return InstrumentsPayload(self.instruments)


import datetime
import sqlite3

from popoll_backend.model import Payload
from popoll_backend.query import Query
from popoll_backend.query.get_session import GetSession


class CreateSession(Query):
    
    id: int
    
    def __init__(self, poll: str, id: str, user_id: int):
        super().__init__(poll)
        self.id = id
        self.user_id = user_id

    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('DELETE FROM sessions WHERE session_id=?', (self.id,))
        cursor.execute('INSERT INTO sessions(session_id, user_id, datetime) VALUES(?, ?, ?)', (self.id, self.user_id, datetime.datetime.now().isoformat(sep='T', timespec='auto')))

    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return GetSession(self.poll, self.id).run(cursor)


from tests import IntegrationTest, TestForwardError

class TestUpdateAnswerEntry(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user2_id = self.create_user('user2', self.instru3_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        
    def test_update_answer_entry(self):
        updated_answer1_id = self.update_answer(self.answer1_id, False)
        assert updated_answer1_id == self.answer1_id
        _json = self.get_user(self.user1_id)
        assert _json['dates'][0]['answer']['response'] == False
        
    # TODO Does not make sens since is returning valid answer but nothing impacted  
    # def test_update_answer_entry_not_found(self):
    #     try:
    #         self.udpate_answer(self.answer1_id + 10, False)
    #         assert False
    #     except TestForwardError as e:
    #         assert e.rs.status_code == 404
    #         _json = self.toJSON(self.get())
    #         assert len(_json['answers']) == 1



import sqlite3
from popoll_backend.model import Payload
from popoll_backend.model.payload.id_payload import IdPayload
from popoll_backend.query import Query


class DeleteUser(Query):
    
    id: int
    
    def __init__(self, poll: str, id: int):
        super().__init__(poll)
        self.id = id
    
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('DELETE FROM users WHERE id=?', (self.id,))
    
    def buildResponse(self, _: sqlite3.Cursor) -> Payload:
        return IdPayload(self.id)
    


from tests import IntegrationTest

class TestDeleteDatabase(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])


    def test_delete_database(self):            
        _json = self.get_users()
        assert len(_json['users']) == 1
        self.delete_database()
        rs = self.get_users(fail=True)
        assert rs.status_code == 400
        




import sqlite3
from typing import List

import flask

from popoll_backend.model import Payload
from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.option import Option
from popoll_backend.model.db.session import Session
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model.payload.id_payload import IdPayload
from popoll_backend.query import Query


class CreatePoll(Query):
    
    fail_if_db_exists: bool = True
    fail_if_db_not_exists: bool = False
    
    name: str
    instruments: List[str]
    color: str
    
    def __init__(self, poll:str, name: str, instruments: List[str], color: str):
        super().__init__(poll)
        self.name = name
        self.instruments = instruments
        self.color = color
    
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute(Option.create_table())
        cursor.execute('INSERT INTO options(name, color) values(?, ?)', (self.name, self.color))
        cursor.execute(Instrument.create_table())
        cursor.executemany('INSERT INTO instruments(name) values(?)', [(d,) for d in self.instruments])
        cursor.execute(Date.create_table())
        cursor.execute(User.create_table())
        cursor.execute(Answer.create_table())
        cursor.execute(UserInstruments.create_table())
        cursor.execute(Session.create_table())
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return IdPayload(0)


import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model.payload.user import UserOut
from popoll_backend.model.payload.users_payload import UsersPayload
from popoll_backend.model.payload.users_payload_details import UsersPayloadDetails
from popoll_backend.query import Query


class GetUsers(Query):
    
    details: bool
    
    users: List[User]
    instruments: List[Instrument]
    user_instruments: List[UserInstruments]
    
    def __init__(self, poll: str, details: bool = False):
        super().__init__(poll)
        self.details = details
    
    def process(self, cursor: sqlite3.Cursor):
        self.users = [User(data) for data in cursor.execute('SELECT * from users ORDER BY name COLLATE NOCASE').fetchall()]       
        if self.details:
            self.instruments: List[Instrument] = [Instrument(data) for data in cursor.execute('SELECT * from instruments').fetchall()]
            self.user_instruments: List[UserInstruments] = [UserInstruments(data) for data in cursor.execute('SELECT * from user_instruments').fetchall()]

    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        if not self.details:
            return UsersPayload(self.users)
        users_out = [UserOut(_user, self.instruments, self.user_instruments) for _user in self.users]    
        return UsersPayloadDetails(users_out)



from typing import List
from popoll_backend.model import Payload
from popoll_backend.model.payload.user import UserOut


class UsersPayloadDetails(Payload):
    
    def __init__(self, users: List[UserOut]):
        self.users: List[UserOut] = users


import datetime
import sqlite3
from typing import Optional

from popoll_backend.model import Payload
from popoll_backend.query import Query
from popoll_backend.query.get_date import GetDate


class UpdateDate(Query):
    
    id: int
    title: str
    date: datetime.date
    time: Optional[datetime.time]
    
    def __init__(self, poll: str, id: int, title: str, date: datetime.date, time: Optional[datetime.time]):
        super().__init__(poll)
        self.id = id
        self.title = title
        self.date = date
        self.time = time
        
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('UPDATE dates SET title=?, date=?, time=? WHERE id=?', (self.title, self.date, self.time, self.id))
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return GetDate(self.poll, self.id, False).run(cursor)


import sqlite3
from typing import List

import flask

from popoll_backend.model import Payload
from popoll_backend.model.db.answer import Answer
from popoll_backend.query import Query


class GetSearchAnswer(Query):
    
    userId: int
    dateId: int
    
    def __init__(self, poll: str, userId: int, dateId: int):
        super().__init__(poll)
        self.userId = userId
        self.dateId = dateId

    def process(self, cursor: sqlite3.Cursor):
        pass
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        res = cursor.execute('SELECT * FROM answers WHERE user_id=? AND date_id=?', (self.userId, self.dateId)).fetchall()
        if (len(res) == 0):
            flask.abort(404, f'Object of type [Answer] and matching user_id=[{self.userId}] and date_id=[{self.dateId}] has not been found.')
        return Answer(res[0])


import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model.payload.user import UserOut
from popoll_backend.model.payload.user_payload import UserPayload
from popoll_backend.query import Query
from popoll_backend.query.get_instruments import GetInstruments


class GetUser(Query):
    
    id: int
    details: bool
    
    user: User
    instruments: List[Instrument]
    user_instruments: List[UserInstruments]
    answers: List[Answer]
    dates: List[Date]
    
    def __init__(self, poll: str, id: int, details: bool = False):
        super().__init__(poll)
        self.id = id
        self.details = details
    
    def process(self, cursor: sqlite3.Cursor):
        self.user = self.selectItem(cursor, 'SELECT * from users WHERE id=?', self.id, User)
        self.instruments: List[Instrument] = GetInstruments(self.poll).run(cursor).instruments
        self.user_instruments: List[UserInstruments] = [UserInstruments(data) for data in cursor.execute('SELECT * from user_instruments WHERE user_id=?', (self.id,)).fetchall()]
        if self.details:
            self.answers: List[Answer] = [Answer(data) for data in cursor.execute('SELECT * FROM answers WHERE user_id=?', (self.id,)).fetchall()]
            self.dates: List[Date] = [Date(data) for data in cursor.execute(f'SELECT * FROM dates ORDER BY date, time').fetchall()]
        
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        user_out: UserOut = UserOut(self.user, self.instruments, self.user_instruments)
        if not self.details:
            return user_out
        return UserPayload(user_out, self.answers, self.dates)
        
        
        


from typing import Any, List

from popoll_backend.model import Payload


class Instrument(Payload):
    
    def __init__(self, data: List[Any]):
        self.id: int = data[0]
        self.name: str = data[1]

    @classmethod
    def create_table(cls):
        return """CREATE TABLE IF NOT EXISTS instruments (
            id integer PRIMARY KEY,
            name text NOT NULL
        );"""


from tests import IntegrationTest

class TestSimpleAdds(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        
    def test_simple_adds(self):
        assert self.user1_id != None
        assert self.date1_id != None
        assert self.answer1_id != None
        _json = self.get_user(self.user1_id)
        assert _json['user']['user']['id'] == self.user1_id
        assert _json['user']['user']['name'] == 'user1'
        assert _json['user']['main_instrument']['id'] == self.instru1_id
        assert len(_json['user']['instruments']) == 0
        assert len(_json['dates']) == 1
        assert _json['dates'][0]['date']['id'] == self.date1_id
        assert _json['dates'][0]['date']['title'] == 'firstDate'
        assert _json['dates'][0]['date']['date'] == '2025-03-10'
        assert _json['dates'][0]['date']['time'] == '15:00:00'
        assert _json['dates'][0]['answer']['id'] == self.answer1_id
        assert _json['dates'][0]['answer']['response'] == True
        